@@ -23,6 +23,7 @@
 #define _SYNAPTICSSTR_H_
 
 #include "synproto.h"
+#include <xserver-properties.h>
 
 #ifdef DBG
 #  undef DBG
@@ -39,22 +40,36 @@
 #define xf86IDrvMsg(pInfo, type, ...) xf86Msg(type, __VA_ARGS__)
 #endif
 
+#ifdef AXIS_LABEL_PROP_REL_VSCROLL
+#define HAVE_SMOOTH_SCROLL
+#endif
+
 /******************************************************************************
  *		Definitions
  *					structs, typedefs, #defines, enums
  *****************************************************************************/
 #define SYNAPTICS_MOVE_HISTORY	5
+#define SYNAPTICS_MAX_TOUCHES	10
 
 typedef struct _SynapticsMoveHist
 {
     int x, y;
-    int millis;
+    CARD32 millis;
 } SynapticsMoveHistRec;
 
-enum FingerState {		/* Note! The order matters. Compared with < operator. */
-    FS_UNTOUCHED,
-    FS_TOUCHED,
-    FS_PRESSED
+typedef struct _SynapticsTouchAxis
+{
+    const char *label;
+    int min;
+    int max;
+    int res;
+} SynapticsTouchAxisRec;
+
+enum FingerState {              /* Note! The order matters. Compared with < operator. */
+    FS_BLOCKED = -1,
+    FS_UNTOUCHED = 0, /* this is 0 so it's the initialized value. */
+    FS_TOUCHED = 1,
+    FS_PRESSED = 2,
 };
 
 enum MovingState {
@@ -85,7 +100,8 @@ enum TapState {
     TS_3,			/* After second touch */
     TS_DRAG,			/* Pointer drag enabled */
     TS_4,			/* After release when "locked drags" enabled */
-    TS_5			/* After touch when "locked drags" enabled */
+    TS_5,			/* After touch when "locked drags" enabled */
+    TS_CLICKPAD_MOVE,		/* After left button press on a clickpad */
 };
 
 enum TapButtonState {
@@ -98,7 +114,8 @@ enum TouchpadModel {
     MODEL_UNKNOWN = 0,
     MODEL_SYNAPTICS,
     MODEL_ALPS,
-    MODEL_APPLETOUCH
+    MODEL_APPLETOUCH,
+    MODEL_ELANTECH
 };
 
 typedef struct _SynapticsParameters
@@ -111,6 +128,7 @@ typedef struct _SynapticsParameters
     int single_tap_timeout;		    /* timeout to recognize a single tap */
     int tap_time_2;			    /* max. tapping time for double taps */
     int click_time;			    /* The duration of a single click */
+    Bool clickpad;                          /* Device is a has integrated buttons */
     Bool fast_taps;			    /* Faster reaction to single taps */
     int emulate_mid_button_time;	    /* Max time between left and right button presses to
 					       emulate a middle button press. */
@@ -164,11 +182,12 @@ typedef struct _SynapticsParameters
     unsigned int resolution_horiz;          /* horizontal resolution of touchpad in units/mm */
     unsigned int resolution_vert;           /* vertical resolution of touchpad in units/mm */
     int area_left_edge, area_right_edge, area_top_edge, area_bottom_edge; /* area coordinates absolute */
+    int softbutton_areas[2][4];             /* soft button area coordinates, 0 => right, 1 => middle button */
     int hyst_x, hyst_y;                     /* x and y width of hysteresis box */
 } SynapticsParameters;
 
 
-typedef struct _SynapticsPrivateRec
+struct _SynapticsPrivateRec
 {
     SynapticsParameters synpara;            /* Default parameter settings, read from
 					       the X config file */
@@ -177,27 +196,42 @@ typedef struct _SynapticsPrivateRec
     struct SynapticsProtocolOperations* proto_ops;
     void *proto_data;			/* protocol-specific data */
 
-    struct SynapticsHwState hwState;
+    struct SynapticsHwState *hwState;
+    struct SynapticsHwState *old_hw_state; /* previous logical hw state */
 
-    char *device;			/* device node */
+    const char *device;			/* device node */
     Bool shm_config;			/* True when shared memory area allocated */
 
+    CARD32 timer_time;			/* when timer last fired */
     OsTimerPtr timer;			/* for up/down-button repeat, tap processing, etc */
 
     struct CommData comm;
 
+    struct SynapticsHwState *local_hw_state; /* used in place of local hw state variables */
+
     Bool absolute_events;               /* post absolute motion events instead of relative */
     SynapticsMoveHistRec move_hist[SYNAPTICS_MOVE_HISTORY]; /* movement history */
     int hist_index;			/* Last added entry in move_hist[] */
     int hyst_center_x;			/* center x of hysteresis*/
     int hyst_center_y;			/* center y of hysteresis*/
-    int scroll_y;			/* last y-scroll position */
-    int scroll_x;			/* last x-scroll position */
-    double scroll_a;			/* last angle-scroll position */
+    struct {
+        int last_x;			/* last x-scroll position */
+        int last_y;			/* last y-scroll position */
+        double delta_x;			/* accumulated horiz scroll delta */
+        double delta_y;			/* accumulated vert scroll delta */
+        double last_a;			/* last angle-scroll position */
+        CARD32 last_millis;		/* time last scroll event posted */
+        double coast_speed_x;		/* Horizontal coasting speed */
+        double coast_speed_y;		/* Vertical coasting speed */
+        double coast_delta_x;		/* Accumulated horizontal coast delta */
+        double coast_delta_y;		/* Accumulated vertical coast delta */
+        int packets_this_scroll;	/* Events received for this scroll */
+    } scroll;
     int count_packet_finger;		/* packet counter with finger on the touchpad */
     int button_delay_millis;		/* button delay for 3rd button emulation */
     Bool prev_up;			/* Previous up button value, for double click emulation */
     enum FingerState finger_state;	/* previous finger state */
+    CARD32 last_motion_millis;	        /* time of the last motion */
 
     enum TapState tap_state;		/* State of tap processing */
     int tap_max_fingers;		/* Max number of fingers seen since entering start state */
@@ -215,18 +249,11 @@ typedef struct _SynapticsPrivateRec
 					   False: Generate horizontal events */
     int trackstick_neutral_x;		/* neutral x position for trackstick mode */
     int trackstick_neutral_y;		/* neutral y position for trackstick mode */
-    double autoscroll_xspd;		/* Horizontal coasting speed */
-    double autoscroll_yspd;		/* Vertical coasting speed */
-    double autoscroll_x;		/* Accumulated horizontal coasting scroll */
-    double autoscroll_y;		/* Accumulated vertical coasting scroll */
-    int scroll_packet_count;		/* Scroll duration */
     double frac_x, frac_y;		/* absolute -> relative fraction */
     enum MidButtonEmulation mid_emu_state;	/* emulated 3rd button */
     int repeatButtons;			/* buttons for repeat */
     int nextRepeat;			/* Time when to trigger next auto repeat event */
     int lastButtons;			/* last state of the buttons */
-    int palm;				/* Set to true when palm detected, reset to false when
-					   palm/finger contact disappears */
     int prev_z;				/* previous z value, for palm detection */
     int prevFingers;			/* previous numFingers, for transition detection */
     int avg_width;			/* weighted average of previous fingerWidth values */
@@ -244,10 +271,27 @@ typedef struct _SynapticsPrivateRec
     Bool has_pressure;			/* device reports pressure */
     Bool has_width;			/* device reports finger width */
     Bool has_scrollbuttons;		/* device has physical scrollbuttons */
+    Bool has_semi_mt;			/* device is only semi-multitouch capable */
 
     enum TouchpadModel model;		/* The detected model */
     unsigned short id_vendor;		/* vendor id */
     unsigned short id_product;		/* product id */
-} SynapticsPrivate;
+
+#ifdef HAVE_SMOOTH_SCROLL
+    int scroll_axis_horiz;              /* Horizontal smooth-scrolling axis */
+    int scroll_axis_vert;               /* Vertical smooth-scrolling axis */
+    ValuatorMask *scroll_events_mask;   /* ValuatorMask for smooth-scrolling */
+#endif
+
+#ifdef HAVE_MULTITOUCH
+    Bool has_touch;                     /* Device has multitouch capabilities */
+    int max_touches;                    /* Number of touches supported */
+    int num_mt_axes;                    /* Number of multitouch axes other than X, Y */
+    SynapticsTouchAxisRec *touch_axes;  /* Touch axis information other than X, Y */
+    int num_slots;                      /* Number of touch slots allocated */
+    int *open_slots;                    /* Array of currently open touch slots */
+    int num_active_touches;             /* Number of active touches on device */
+#endif
+};
 
 #endif /* _SYNAPTICSSTR_H_ */
