@@ -38,6 +38,7 @@
 #include <string.h>
 #include <stddef.h>
 #include <math.h>
+#include <limits.h>
 
 #include <X11/Xdefs.h>
 #include <X11/Xatom.h>
@@ -45,7 +46,6 @@
 #include <X11/extensions/XInput.h>
 #include "synaptics.h"
 #include "synaptics-properties.h"
-#include <xserver-properties.h>
 
 #ifndef XATOM_FLOAT
 #define XATOM_FLOAT "FLOAT"
@@ -90,15 +90,15 @@ static struct Parameter params[] = {
     {"EmulateMidButtonTime",  PT_INT,    0, 1000,  SYNAPTICS_PROP_MIDDLE_TIMEOUT,32,	0},
     {"EmulateTwoFingerMinZ",  PT_INT,    0, 1000,  SYNAPTICS_PROP_TWOFINGER_PRESSURE,	32,	0},
     {"EmulateTwoFingerMinW",  PT_INT,    0, 15,    SYNAPTICS_PROP_TWOFINGER_WIDTH,	32,	0},
-    {"VertScrollDelta",       PT_INT,    0, 1000,  SYNAPTICS_PROP_SCROLL_DISTANCE,	32,	0},
-    {"HorizScrollDelta",      PT_INT,    0, 1000,  SYNAPTICS_PROP_SCROLL_DISTANCE,	32,	1},
+    {"VertScrollDelta",       PT_INT,    -1000, 1000,  SYNAPTICS_PROP_SCROLL_DISTANCE,	32,	0},
+    {"HorizScrollDelta",      PT_INT,    -1000, 1000,  SYNAPTICS_PROP_SCROLL_DISTANCE,	32,	1},
     {"VertEdgeScroll",        PT_BOOL,   0, 1,     SYNAPTICS_PROP_SCROLL_EDGE,	8,	0},
     {"HorizEdgeScroll",       PT_BOOL,   0, 1,     SYNAPTICS_PROP_SCROLL_EDGE,	8,	1},
     {"CornerCoasting",        PT_BOOL,   0, 1,     SYNAPTICS_PROP_SCROLL_EDGE,	8,	2},
     {"VertTwoFingerScroll",   PT_BOOL,   0, 1,     SYNAPTICS_PROP_SCROLL_TWOFINGER,	8,	0},
     {"HorizTwoFingerScroll",  PT_BOOL,   0, 1,     SYNAPTICS_PROP_SCROLL_TWOFINGER,	8,	1},
-    {"MinSpeed",              PT_DOUBLE, 0, 1.0,   SYNAPTICS_PROP_SPEED,	0, /*float */	0},
-    {"MaxSpeed",              PT_DOUBLE, 0, 1.0,   SYNAPTICS_PROP_SPEED,	0, /*float */	1},
+    {"MinSpeed",              PT_DOUBLE, 0, 255.0,   SYNAPTICS_PROP_SPEED,	0, /*float */	0},
+    {"MaxSpeed",              PT_DOUBLE, 0, 255.0,   SYNAPTICS_PROP_SPEED,	0, /*float */	1},
     {"AccelFactor",           PT_DOUBLE, 0, 1.0,   SYNAPTICS_PROP_SPEED,	0, /*float */	2},
     {"TrackstickSpeed",       PT_DOUBLE, 0, 200.0, SYNAPTICS_PROP_SPEED,	0, /*float */ 3},
     {"EdgeMotionMinZ",        PT_INT,    1, 255,   SYNAPTICS_PROP_EDGEMOTION_PRESSURE,  32,	0},
@@ -112,7 +112,6 @@ static struct Parameter params[] = {
     {"LeftRightScrollRepeat", PT_BOOL,   0, 1,     SYNAPTICS_PROP_BUTTONSCROLLING_REPEAT,   8,	1},
     {"ScrollButtonRepeat",    PT_INT,    SBR_MIN , SBR_MAX, SYNAPTICS_PROP_BUTTONSCROLLING_TIME, 32,	0},
     {"TouchpadOff",           PT_INT,    0, 2,     SYNAPTICS_PROP_OFF,		8,	0},
-    {"GuestMouseOff",         PT_BOOL,   0, 1,     SYNAPTICS_PROP_GUESTMOUSE,	8,	0},
     {"LockedDrags",           PT_BOOL,   0, 1,     SYNAPTICS_PROP_LOCKED_DRAGS,	8,	0},
     {"LockedDragTimeout",     PT_INT,    0, 30000, SYNAPTICS_PROP_LOCKED_DRAGS_TIMEOUT,	32,	0},
     {"RTCornerButton",        PT_INT,    0, SYN_MAX_BUTTONS, SYNAPTICS_PROP_TAP_ACTION,	8,	0},
@@ -133,6 +132,7 @@ static struct Parameter params[] = {
     {"PalmMinWidth",          PT_INT,    0, 15,    SYNAPTICS_PROP_PALM_DIMENSIONS,	32,	0},
     {"PalmMinZ",              PT_INT,    0, 255,   SYNAPTICS_PROP_PALM_DIMENSIONS,	32,	1},
     {"CoastingSpeed",         PT_DOUBLE, 0, 20,    SYNAPTICS_PROP_COASTING_SPEED,	0 /* float*/,	0},
+    {"CoastingFriction",      PT_DOUBLE, 0, 255,   SYNAPTICS_PROP_COASTING_SPEED,	0 /* float*/,	1},
     {"PressureMotionMinZ",    PT_INT,    1, 255,   SYNAPTICS_PROP_PRESSURE_MOTION,	32,	0},
     {"PressureMotionMaxZ",    PT_INT,    1, 255,   SYNAPTICS_PROP_PRESSURE_MOTION,	32,	1},
     {"PressureMotionMinFactor", PT_DOUBLE, 0, 10.0,SYNAPTICS_PROP_PRESSURE_MOTION_FACTOR,	0 /*float*/,	0},
@@ -143,13 +143,24 @@ static struct Parameter params[] = {
     {"AreaRightEdge",         PT_INT,    0, 10000, SYNAPTICS_PROP_AREA,	32,	1},
     {"AreaTopEdge",           PT_INT,    0, 10000, SYNAPTICS_PROP_AREA,	32,	2},
     {"AreaBottomEdge",        PT_INT,    0, 10000, SYNAPTICS_PROP_AREA,	32,	3},
+    {"HorizHysteresis",       PT_INT,    0, 10000, SYNAPTICS_PROP_NOISE_CANCELLATION, 32,	0},
+    {"VertHysteresis",        PT_INT,    0, 10000, SYNAPTICS_PROP_NOISE_CANCELLATION, 32,	1},
+    {"ClickPad",              PT_BOOL,   0, 1,     SYNAPTICS_PROP_CLICKPAD,	8,	0},
+    {"RightButtonAreaLeft",   PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	0},
+    {"RightButtonAreaRight",  PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	1},
+    {"RightButtonAreaTop",    PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	2},
+    {"RightButtonAreaBottom", PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	3},
+    {"MiddleButtonAreaLeft",  PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	4},
+    {"MiddleButtonAreaRight", PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	5},
+    {"MiddleButtonAreaTop",   PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	6},
+    {"MiddleButtonAreaBottom", PT_INT, INT_MIN, INT_MAX, SYNAPTICS_PROP_SOFTBUTTON_AREAS,	32,	7},
     { NULL, 0, 0, 0, 0 }
 };
 
 static double
 parse_cmd(char* cmd, struct Parameter** par)
 {
-    char *eqp = index(cmd, '=');
+    char *eqp = strchr(cmd, '=');
     *par = NULL;
 
     if (eqp) {
@@ -196,12 +207,7 @@ is_equal(SynapticsSHM *s1, SynapticsSHM *s2)
 	(s1->right       != s2->right) ||
 	(s1->up          != s2->up) ||
 	(s1->down        != s2->down) ||
-	(s1->middle      != s2->middle) ||
-	(s1->guest_left  != s2->guest_left) ||
-	(s1->guest_mid   != s2->guest_mid) ||
-	(s1->guest_right != s2->guest_right) ||
-	(s1->guest_dx    != s2->guest_dx) ||
-	(s1->guest_dy    != s2->guest_dy))
+	(s1->middle      != s2->middle))
 	return 0;
 
     for (i = 0; i < 8; i++)
@@ -240,15 +246,12 @@ shm_monitor(SynapticsSHM *synshm, int delay)
 		header = 20;
 	    }
 	    header--;
-	    printf("%8.3f  %4d %4d %3d %d %2d %2d %d %d %d %d  %d%d%d%d%d%d%d%d  "
-		   "%2d %2d %2d %3d %3d\n",
+	    printf("%8.3f  %4d %4d %3d %d %2d %2d %d %d %d %d  %d%d%d%d%d%d%d%d\n",
 		   get_time() - t0,
 		   cur.x, cur.y, cur.z, cur.numFingers, cur.fingerWidth,
 		   cur.left, cur.right, cur.up, cur.down, cur.middle,
 		   cur.multi[0], cur.multi[1], cur.multi[2], cur.multi[3],
-		   cur.multi[4], cur.multi[5], cur.multi[6], cur.multi[7],
-		   cur.guest_left, cur.guest_mid, cur.guest_right,
-		   cur.guest_dx, cur.guest_dy);
+		   cur.multi[4], cur.multi[5], cur.multi[6], cur.multi[7]);
 	    fflush(stdout);
 	    old = cur;
 	}
@@ -443,6 +446,13 @@ dp_set_variables(Display *dpy, XDevice* dev, int argc, char *argv[], int first_c
 	XGetDeviceProperty(dpy, dev, prop, 0, 1000, False, AnyPropertyType,
 				&type, &format, &nitems, &bytes_after, &data);
 
+	if (type == None)
+	{
+	    fprintf(stderr, "Property for '%s' not available. Skipping.\n",
+		    par->name);
+	    continue;
+	}
+
 	switch(par->prop_format)
 	{
 	    case 8:
@@ -455,7 +465,8 @@ dp_set_variables(Display *dpy, XDevice* dev, int argc, char *argv[], int first_c
 		b[par->prop_offset] = rint(val);
 		break;
 	    case 32:
-		if (format != par->prop_format || type != XA_INTEGER) {
+		if (format != par->prop_format ||
+                    (type != XA_INTEGER && type != XA_CARDINAL)) {
 		    fprintf(stderr, "   %-23s = format mismatch (%d)\n",
 			    par->name, format);
 		    break;
@@ -505,22 +516,21 @@ dp_show_settings(Display *dpy, XDevice *dev)
     for (j = 0; params[j].name; j++) {
 	struct Parameter *par = &params[j];
 	a = XInternAtom(dpy, par->prop_name, True);
-	if (!a) {
-	    fprintf(stderr, "    %-23s = missing\n",
-		    par->name);
+	if (!a)
 	    continue;
-	}
 
 	len = 1 + ((par->prop_offset * (par->prop_format ? par->prop_format : 32)/8))/4;
 
 	XGetDeviceProperty(dpy, dev, a, 0, len, False,
 				AnyPropertyType, &type, &format,
 				&nitems, &bytes_after, &data);
+	if (type == None)
+	    continue;
 
 	switch(par->prop_format) {
 	    case 8:
 		if (format != par->prop_format || type != XA_INTEGER) {
-		    fprintf(stderr, "   %-23s = format mismatch (%d)\n",
+		    fprintf(stderr, "    %-23s = format mismatch (%d)\n",
 			    par->name, format);
 		    break;
 		}
@@ -529,8 +539,9 @@ dp_show_settings(Display *dpy, XDevice *dev)
 		printf("    %-23s = %d\n", par->name, b[par->prop_offset]);
 		break;
 	    case 32:
-		if (format != par->prop_format || type != XA_INTEGER) {
-		    fprintf(stderr, "   %-23s = format mismatch (%d)\n",
+		if (format != par->prop_format ||
+                    (type != XA_INTEGER && type != XA_CARDINAL)) {
+		    fprintf(stderr, "    %-23s = format mismatch (%d)\n",
 			    par->name, format);
 		    break;
 		}
@@ -542,7 +553,7 @@ dp_show_settings(Display *dpy, XDevice *dev)
 		if (!float_type)
 		    continue;
 		if (format != 32 || type != float_type) {
-		    fprintf(stderr, "   %-23s = format mismatch (%d)\n",
+		    fprintf(stderr, "    %-23s = format mismatch (%d)\n",
 			    par->name, format);
 		    break;
 		}
@@ -581,6 +592,9 @@ main(int argc, char *argv[])
     Display *dpy;
     XDevice *dev;
 
+    if (argc == 1)
+        dump_settings = 1;
+
     /* Parse command line parameters */
     while ((c = getopt(argc, argv, "sm:hlV")) != -1) {
 	switch (c) {
